/* 
**
**  remedial - a GUI front-end for avifile
**  Copyright (C) 2002  Paul Dwerryhouse <paul@dwerryhouse.com.au>
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License version 2, as 
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


extern "C" {
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif
#include <sys/types.h>
#include <dirent.h>
#include <dlfcn.h>
}
#include <cstdio>
#include <qapplication.h>
#include <qtimer.h>
#include "qtgui.h"
#include "qtdock.h"
#include "qtnewgui.h"
#include "player.h"
#include "plugin.h"
extern char *optarg;

void usage(char *);
void version(void);
struct input_plugin *plugins(void);

// Maximum number of plugins allowed.
// This should be rewritten so that it's unlimited.
#define MAX_PLUGINS 10

int main(int argc, char **argv)
{

    int c, preset = -1, dock = 0, oldgui = 0;
    struct input_plugin *in_plugins;

    while ((c = getopt(argc, argv, "hvdop:")) != -1) {
	switch (c) {
	case 'p':
	    preset = atoi(optarg);
	    break;
	case 'v':
	    version();
	    exit(0);
	case 'd':
	    dock = 1;
	    break;
	case 'o':
	    oldgui = 1;
	    break;
	case 'h':		/* Fallthrough */
	default:
	    usage(argv[0]);
	    break;
	}
    }

    version();
    in_plugins = plugins();

    QApplication a(argc, argv);

    Player *player = new Player();
    player->setPlugins(in_plugins);

    if (dock) {
	QtDock dockw;

	dockw.setPlayer(player);
	dockw.show();
	a.setMainWidget(&dockw);
	return a.exec();
    } else if (oldgui) {
	QtGui widget;
	widget.setPlayer(player);
	widget.resize(315, 256);


	a.setMainWidget(&widget);
	widget.show();
	return a.exec();
    }

    if (preset >= 0) {
	player->setPreset(preset);
    } else if (argc == 2) {
	player->setFile(argv[1]);
    }

    QtNewGui newguiw;

    newguiw.setPlayer(player);
    newguiw.show();
    a.setMainWidget(&newguiw);
    return a.exec();


}

void usage(char *prog)
{
    printf
	("Usage: %s [-v] | [-h] | [-d] [-o] | [-p <preset>] | [filename]\n",
	 prog);
    exit(0);
}

void version(void)
{
    printf("Remedial version " VERSION ", Copyright (C) 2002, 2003  "
	   "Paul Dwerryhouse\n"
	   "Remedial comes with ABSOLUTELY NO WARRANTY.\n"
	   "This is free software, and you are welcome to redistribute it\n"
	   "under the conditions of the GNU General Public License, "
	   "version 2.0.\n"
	   "Please see the COPYING file supplied with this program for "
	   "details.\n");
}

struct input_plugin *plugins(void)
{
    DIR *plugdir;
    int i = 0;
    struct dirent *entry;
    struct input_plugin *in_plugins;

    in_plugins = (struct input_plugin *) malloc(MAX_PLUGINS *
						sizeof(struct
						       input_plugin));

    plugdir = opendir(PLUGINDIR);

    // Search plugin directory for plugins
    while (entry = readdir(plugdir)) {
	int len = strlen(entry->d_name);
	if ((entry->d_type & DT_REG) == DT_REG &&
	    entry->d_name[len] == '\0' &&
	    entry->d_name[len - 1] == 'o' &&
	    entry->d_name[len - 2] == 's') {

	    char *plugpath;
	    void *plugin;
	    char *(*plugin_info) (void);
	    char *(*input_type) (void);
	    int (*input_plugin) (void);

	    printf("Found module %s %d\n", entry->d_name,
		   entry->d_type & DT_REG);

	    plugpath = (char *) malloc(sizeof(char) *
				       strlen(PLUGINDIR) +
				       strlen(entry->d_name) + 2);
	    sprintf(plugpath, PLUGINDIR "/%s", entry->d_name);

	    printf("Path: %s\n", plugpath);

	    plugin = dlopen(plugpath, RTLD_LAZY);
	    plugin_info = (char *(*)(void)) dlsym(plugin, "plugin_info");
	    input_plugin = (int (*)(void)) dlsym(plugin, "input_plugin");

	    free(plugpath);

	    printf("Info: %s\n", (*plugin_info) ());
	    if ((*input_plugin) ()) {
		input_type = (char *(*)(void)) dlsym(plugin, "input_type");
		in_plugins[i].type = (*input_type) ();
		in_plugins[i].plugin = plugin;
		i++;
	    }
	}
	if (i >= MAX_PLUGINS)
	    break;
    }

    in_plugins[i].type = 0;
    closedir(plugdir);

    return (struct input_plugin *) in_plugins;
}
