/* 
**
**  remedial - a GUI front-end for avifile
**  Copyright (C) 2002  Paul Dwerryhouse <paul@dwerryhouse.com.au>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License version 2, as 
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


extern "C" {
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dlfcn.h>
}
#include "player.h"
#include "configfile.h"
#ifdef HAVE_LINUX_SOUNDCARD_H
#include <linux/soundcard.h>
#endif
#define FILE "mms://media4.abc.net.au/triplej"
#define GLOBALCFG SYSCONFDIR "/remedial.xml"
#define CFGFILE "/.remedial/remedial.xml"
#ifdef HAVE_LINUX_SOUNDCARD_H
int mixer;
#endif

struct input_plugin *in_plugins;
int use_plugin;
struct preset *readConfig(void);

Player::Player()
{
    char *name;
    int len;

    file = FILE;
    muted = 0;
    autostart = 0;

    presets = readConfig();

#ifdef HAVE_LINUX_SOUNDCARD_H
    // Don't close it! It makes avifile segfault.
    mixer = open("/dev/mixer", O_RDWR);
#endif
}

int Player::start()
{

    int i;
    for (i = 0; in_plugins[i].type != 0; i++) {
	if (!strcmp(in_plugins[i].type, type)) {
	    fprintf(stderr, "Found: %s\n", in_plugins[i].type);
	    break;
	}
    }
    if (in_plugins[i].type != 0) {
	int (*plugin_start) (char *);
	use_plugin = i;

	plugin_start = (int (*)(char *))
	    dlsym(in_plugins[i].plugin, "start");
	(*plugin_start) (file);
    }
}

void Player::stop()
{
    void (*plugin_stop) (void);

    plugin_stop = (void (*)(void))
	dlsym(in_plugins[use_plugin].plugin, "stop");
    (*plugin_stop) ();
}

void Player::buffer()
{
    if (type == 0) {
#ifdef XXX
	double x;
	avm::IAviPlayer::State state;
	state = player->GetState(&x);
	if (state == avm::IAviPlayer::Buffering)
	    printf("%f\n", (float) x);
#endif
    } else {
    }
}

void Player::setProxy(bool enabled, const char *newproxy)
{
    useproxy = enabled;
    proxy = (char *) newproxy;
}

void Player::mute()
{

    if (muted) {
#ifdef HAVE_LINUX_SOUNDCARD_H
	ioctl(mixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &volume);
#endif
	muted = 0;
    } else {
	unsigned int newvol = 0;
#ifdef HAVE_LINUX_SOUNDCARD_H
	ioctl(mixer, MIXER_READ(SOUND_MIXER_VOLUME), &volume);
	ioctl(mixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &newvol);
#endif
	muted = 1;
    }
}

void Player::setvol(int vol)
{
    unsigned int setvol;

#ifdef HAVE_LINUX_SOUNDCARD_H
    setvol = (vol << 8) + vol;
    ioctl(mixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &setvol);
#endif
}

unsigned int Player::getvol()
{
    unsigned int volume;

#ifdef HAVE_LINUX_SOUNDCARD_H
    unsigned int left, right;
    ioctl(mixer, MIXER_READ(SOUND_MIXER_VOLUME), &volume);
    right = volume & 255;
    left = volume >> 8;
    volume = (left + right) / 2;
#endif

    return volume;
}

void Player::choosepreset(int preset)
{
    file = presets[preset].preseturl;
    type = presets[preset].presettype;
    fprintf(stderr, ">>> %s\n", presets[preset].presettype);
}

char *Player::getname(int preset)
{
    return presets[preset].presetname;
}

int Player::status()
{
    int (*plugin_status) (void);

    plugin_status = (int (*)(void))
	dlsym(in_plugins[use_plugin].plugin, "status");
    (*plugin_status) ();
}

void Player::setPlugins(struct input_plugin *p)
{
    in_plugins = p;
}

void Player::rereadConfig()
{
    free_presets(presets);
    presets = readConfig();
}

struct preset *readConfig(void)
{
    char *name;
    int len;

    struct preset *tmp;

    name = strdup(getenv("HOME"));
    len = strlen(name);
    name = (char *) realloc(name, (len + strlen(CFGFILE) + 1)
			    * sizeof(char));
    strcat(name, CFGFILE);

    if (!(tmp = readconf(name))) {
	if (!(tmp = readconf(GLOBALCFG))) {
	    if (!(tmp = readconf("./remedial.xml"))) {
		fprintf(stderr, "Can't find config file\n");
		exit(1);
	    }
	}
    }

    free(name);

    return tmp;
}
